import type { RobotGroup, RobotInfo } from '@api/robot';
import type { Meta2dData } from '@meta2d/core';

export interface SceneInfo {
  id: string; // 场景id
  label: string; // 场景名称
}
export interface SceneDetail extends SceneInfo {
  json?: string; // 场景JSON
}
export interface GroupSceneDetail extends SceneDetail {
  group: RobotGroup; // 机器人组信息
  robots?: Array<RobotInfo>; // 机器人信息
}

export interface SceneData extends Meta2dData {
  robotGroups?: Array<RobotGroup>; // 机器人组信息
  robots?: Array<RobotInfo>; // 机器人信息
}

export interface StandardScene {
  scale?: number; // 缩放比例
  origin?: { x: number; y: number }; // 默认载入原点
  robotGroups?: Array<RobotGroup>; // 机器人组信息
  robots?: Array<RobotInfo>; // 机器人信息
  points?: Array<StandardScenePoint>; // 标准点位信息
  routes?: Array<StandardSceneRoute>; // 标准线路信息
  areas?: Array<StandardSceneArea>; // 标准区域信息
  blocks?: Array<[number, number]>; // 障碍点集合
  width?: number; // 场景宽度
  height?: number; // 场景高度
  ratio?: number; // 坐标缩放比例
}
export interface StandardScenePoint {
  id: string;
  name: string;
  desc?: string; // 描述
  x: number;
  y: number;
  type: number; // 点位类型
  extensionType?: number; // 扩展类型
  robots?: Array<string>; // 绑定机器人id集合
  actions?: Array<string>; // 绑定动作点id集合
  associatedStorageLocations?: string[]; // 库位名称
  config?: object; // 其它属性配置（可按需增加）
  properties?: unknown; // 附加数据（前端不做任何处理）
  deviceId?: string; // 设备ID
}
export interface StandardSceneRoute {
  id: string;
  desc?: string; // 描述
  from: string; // 起点点位id
  to: string; // 终点点位id
  type: 'line' | 'bezier2' | 'bezier3'; // 线路类型
  pass?: number; // 可通行类型
  c1?: { x?: number; y?: number }; // 控制点1
  c2?: { x?: number; y?: number }; // 控制点2
  config?: object; // 其它属性配置（可按需增加）
  properties?: unknown; // 附加数据（前端不做任何处理）
}
export interface StandardSceneArea {
  id: string;
  name: string;
  desc?: string; // 描述
  x: number;
  y: number;
  w: number;
  h: number;
  type: number; // 区域类型
  points?: Array<string>; // 绑定点位id集合
  routes?: Array<string>; // 绑定线路id集合
  maxAmr?: number; // 最大可容纳AMR数
  inoutflag?: 1 | 2; // 库区规则
  storageLocations?: Array<Record<string, string[]>>; // 动作点对应的库位信息，格式为 [{动作点名称: [库位列表]}]
  config?: object; // 其它属性配置（可按需增加）
  properties?: unknown; // 附加数据（前端不做任何处理）
}

// 库位状态相关类型定义
export interface StorageLocationInfo {
  id: string; // 层ID
  layer_index: number; // 层索引(从1开始)
  layer_name: string; // 层名称、库位名称
  operate_point_id: string; // 动作点ID
  station_name: string; // 站点名称
  scene_id: string; // 场景ID
  storage_area_id: string; // 库区ID
  area_name: string; // 库区名称
  is_occupied: boolean; // 是否占用
  is_locked: boolean; // 是否锁定
  is_disabled: boolean; // 是否禁用
  is_empty_tray: boolean; // 是否空托盘
  locked_by: string | null; // 锁定者
  goods_content: string; // 货物内容
  goods_weight: number | null; // 货物重量(克)
  goods_volume: number | null; // 货物体积(立方厘米)
  goods_stored_at: string | null; // 货物存放时间
  goods_retrieved_at: string | null; // 货物取出时间
  last_access_at: string; // 最后访问时间
  max_weight: number; // 最大承重(克)
  max_volume: number; // 最大体积(立方厘米)
  layer_height: number; // 层高(毫米)
  tags: string; // 标签
  description: string | null; // 层描述
  created_at: string; // 创建时间
  updated_at: string; // 更新时间
}

// WebSocket消息类型
export interface StorageLocationUpdateMessage {
  type: 'storage_location_update';
  scene_id: string;
  timestamp: string;
  message: string;
  data: {
    total: number;
    page: number;
    page_size: number;
    total_pages: number;
    storage_locations: StorageLocationInfo[];
  };
}

export interface StorageLocationStatusChangeMessage {
  type: 'storage_location_status_change';
  scene_id: string;
  layer_name: string;
  action: string;
  timestamp: string;
  new_status: {
    id: string;
    is_occupied: boolean;
    is_locked: boolean;
    is_disabled: boolean;
    is_empty_tray: boolean;
    locked_by: string | null;
    goods_content: string;
    last_access_at: string;
    updated_at: string;
  };
}

export interface StorageLocationErrorMessage {
  type: 'error';
  scene_id: string;
  timestamp: string;
  message: string;
}

export type StorageLocationMessage =
  | StorageLocationUpdateMessage
  | StorageLocationStatusChangeMessage
  | StorageLocationErrorMessage;

// 客户端发送消息类型
export interface StorageLocationClientMessage {
  type: 'get_status';
  timestamp: string;
}
