import { ref } from 'vue';

import type { EditorService } from './editor.service';

/**
 * 画布视图状态接口
 */
interface ViewState {
  /** 缩放比例 */
  scale: number;
  /** 中心点X坐标 */
  centerX: number;
  /** 中心点Y坐标 */
  centerY: number;
  /** 保存时间戳 */
  timestamp: number;
}

/**
 * 画布视图状态管理Hook
 * 用于保存和恢复画布的缩放比例和中心坐标
 */
export function useViewState() {
  const isSaving = ref(false);
  const isRestoring = ref(false);

  /**
   * 生成localStorage的key
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const getStorageKey = (sceneId: string, groupId?: string): string => {
    return groupId ? `view-state-${groupId}-${sceneId}` : `view-state-${sceneId}`;
  };

  /**
   * 获取当前画布的视图状态
   * @param editor 编辑器服务实例
   */
  const getCurrentViewState = (editor: EditorService): ViewState => {
    // 获取当前缩放比例和中心点坐标
    const scale = editor.store.data.scale || 1;
    const { centerX, centerY } = calculateCenterPoint(editor);

    const finalState: ViewState = {
      scale,
      centerX,
      centerY,
      timestamp: Date.now(),
    };

    return finalState;
  };

  /**
   * 计算画布中心点坐标
   * @param editor 编辑器服务实例
   */
  const calculateCenterPoint = (editor: EditorService): { centerX: number; centerY: number } => {
    // 1. 查找画布上所有的点位
    const points = editor.find('point');

    // 如果没有点位，返回当前视图中心
    if (!points.length) {
      return {
        centerX: 0,
        centerY: 0,
      };
    }

    // 2. 获取场景数据
    const sceneData = getSceneData(editor);
    const { ratio = 1, width: mapWidth = 0, height: mapHeight = 0 } = sceneData;

    // 3. 计算所有点位的边界
    let minX = Infinity;
    let minY = Infinity;
    let maxX = -Infinity;
    let maxY = -Infinity;

    points.forEach((point) => {
      // 获取点位的实际坐标（画布上显示的坐标）
      const { x: displayX = 0, y: displayY = 0 } = editor.getPointRect(point) ?? {};

      // 将显示坐标转换回原始坐标系统
      // 1. 从中心点原点转回左上角原点坐标
      const topLeftX = (displayX + mapWidth / 2) * ratio;
      const topLeftY = (mapHeight / 2 - displayY) * ratio;

      // 使用原始坐标系统的坐标计算边界
      minX = Math.min(minX, topLeftX);
      minY = Math.min(minY, topLeftY);
      maxX = Math.max(maxX, topLeftX);
      maxY = Math.max(maxY, topLeftY);
    });

    // 如果没有有效的边界，返回原点
    if (minX === Infinity || minY === Infinity || maxX === -Infinity || maxY === -Infinity) {
      return {
        centerX: 0,
        centerY: 0,
      };
    }

    // 4. 计算边界的中心点（在原始坐标系统中）
    const centerX = (minX + maxX) / 2;
    const centerY = (minY + maxY) / 2;

    // 5. 将中心点从原始坐标系统转换回编辑器的显示坐标系统
    // 先根据ratio进行缩放
    const scaledCenterX = centerX / ratio;
    const scaledCenterY = centerY / ratio;

    // 再进行坐标系转换：左上角原点 -> 中心点原点
    const displayCenterX = scaledCenterX - mapWidth / 2;
    const displayCenterY = mapHeight / 2 - scaledCenterY;

    return {
      centerX: displayCenterX,
      centerY: displayCenterY,
    };
  };

  /**
   * 获取场景数据
   * @param editor 编辑器服务实例
   */
  const getSceneData = (editor: EditorService): { ratio: number; width: number; height: number } => {
    try {
      const sceneJson = editor.save();
      if (sceneJson) {
        const sceneData = JSON.parse(sceneJson);
        return {
          ratio: sceneData.ratio || 1,
          width: sceneData.width || 0,
          height: sceneData.height || 0,
        };
      }
    } catch (error) {
      console.error('获取编辑器原始数据失败:', error);
    }

    // 返回默认值
    return { ratio: 1, width: 0, height: 0 };
  };

  /**
   * 保存当前视图状态到localStorage
   * @param editor 编辑器服务实例
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const saveViewState = async (editor: EditorService, sceneId: string, groupId?: string): Promise<void> => {
    isSaving.value = true;
    try {
      const viewState = getCurrentViewState(editor);
      const storageKey = getStorageKey(sceneId, groupId);
      localStorage.setItem(storageKey, JSON.stringify(viewState));
    } catch (error) {
      console.error('保存视图状态失败:', error);
      throw error;
    } finally {
      isSaving.value = false;
    }
  };

  /**
   * 从localStorage恢复视图状态
   * @param editor 编辑器服务实例
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const restoreViewState = async (editor: EditorService, sceneId: string, groupId?: string): Promise<boolean> => {
    isRestoring.value = true;
    try {
      const storageKey = getStorageKey(sceneId, groupId);
      const savedState = localStorage.getItem(storageKey);

      if (!savedState) {
        return false;
      }

      const viewState: ViewState = JSON.parse(savedState);

      // 1. 设置缩放比例
      editor.scale(viewState.scale);

      // 2. 如果有中心点坐标，创建临时点位并跳转
      if (viewState.centerX !== 0 || viewState.centerY !== 0) {
        await jumpToPosition(editor, viewState.centerX, viewState.centerY);
      }

      return true;
    } catch (error) {
      console.error('恢复视图状态失败:', error);
      return false;
    } finally {
      isRestoring.value = false;
    }
  };

  /**
   * 跳转到指定位置
   * @param editor 编辑器服务实例
   * @param x X坐标
   * @param y Y坐标
   */
  const jumpToPosition = async (editor: EditorService, x: number, y: number): Promise<void> => {
    try {
      // 创建临时点位用于跳转
      const centerPointId = 'view-center-point-' + Date.now();

      // 添加临时点位
      await editor.addPoint(
        { x, y }, // 点位坐标
        1, // 点位类型：普通点
        centerPointId, // 点位ID
      );

      // 设置点位为不可见
      editor.setValue({ id: centerPointId, visible: false }, { render: false, history: false, doEvent: false });

      // 跳转到临时点
      editor.gotoById(centerPointId);

      // 延迟移除临时点
      setTimeout(() => {
        const tempPen = editor.getPenById(centerPointId);
        if (tempPen) {
          editor.delete([tempPen]);
        }
      }, 100);
    } catch (error) {
      console.error('跳转到指定位置失败:', error);
    }
  };

  /**
   * 检查是否存在保存的视图状态
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const hasViewState = (sceneId: string, groupId?: string): boolean => {
    const storageKey = getStorageKey(sceneId, groupId);
    return localStorage.getItem(storageKey) !== null;
  };

  /**
   * 清除保存的视图状态
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const clearViewState = (sceneId: string, groupId?: string): void => {
    const storageKey = getStorageKey(sceneId, groupId);
    localStorage.removeItem(storageKey);
  };

  /**
   * 获取保存的视图状态信息
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const getViewStateInfo = (sceneId: string, groupId?: string): ViewState | null => {
    try {
      const storageKey = getStorageKey(sceneId, groupId);
      const savedState = localStorage.getItem(storageKey);
      return savedState ? JSON.parse(savedState) : null;
    } catch {
      return null;
    }
  };

  /**
   * 自动保存和恢复视图状态
   * 如果本地没有保存的key，则先保存当前状态（用于定位地图），然后恢复
   * 如果有key，则直接按原来逻辑恢复
   * @param editor 编辑器服务实例
   * @param sceneId 场景ID
   * @param groupId 群组ID (可选)
   */
  const autoSaveAndRestoreViewState = async (
    editor: EditorService,
    sceneId: string,
    groupId?: string,
  ): Promise<boolean> => {
    try {
      const hasExistingState = hasViewState(sceneId, groupId);

      if (!hasExistingState) {
        // 先让编辑器居中显示，这样可以获得一个合适的默认视图状态
        editor.centerView();

        // 等待一小段时间让centerView完成
        await new Promise((resolve) => setTimeout(resolve, 100));

        // 保存当前状态作为默认状态
        await saveViewState(editor, sceneId, groupId);

        // 然后恢复这个刚保存的状态（主要是为了触发定位逻辑）
        return await restoreViewState(editor, sceneId, groupId);
      } else {
        // 直接恢复已有状态
        return await restoreViewState(editor, sceneId, groupId);
      }
    } catch (error) {
      console.error('自动保存和恢复视图状态失败:', error);

      // 如果出错，至少执行centerView作为fallback
      editor.centerView();
      return false;
    }
  };

  return {
    isSaving,
    isRestoring,
    saveViewState,
    restoreViewState,
    hasViewState,
    clearViewState,
    getViewStateInfo,
    getCurrentViewState,
    autoSaveAndRestoreViewState,
  };
}
